<?php
declare(strict_types=1);

namespace Catequil\Db\Container;

use Catequil\Db\Connection;
use Doctrine\Common\EventManager;
use Doctrine\DBAL\Connection as DBALConnection;
use Doctrine\DBAL\DriverManager;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Exception\ServiceNotFoundException;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;

use Throwable;
use function is_array;

/**
 * Class ConnectionAbstractFactory
 *
 * @package Catequil\Db\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class ConnectionAbstractFactory implements AbstractFactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     *
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName): bool
    {
        return $requestedName === 'db' || is_array($this->getConfig($container, $requestedName));
    }

    /**
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param array|null $options
     *
     * @return DBALConnection
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): DBALConnection
    {
        $config = $this->getConfig($container, $requestedName);

        if (!isset($config)) {
            throw new ServiceNotFoundException(sprintf(
                "The database with the name '%s' was not found in the container.",
                $requestedName,
            ));
        }

        if (!isset($config['driver'])) {
            $config['driver'] = 'pdo_mysql';
        }

        if (!isset($config['charset'])) {
            $config['charset'] = 'utf8mb4';
        }

        $config['wrapperClass'] = Connection::class;

        try {
            return DriverManager::getConnection(
                $config,
                null,
                $container->get(EventManager::class),
            );
        } catch (Throwable $e) {
            throw new ServiceNotCreatedException($e->getMessage(), $e->getCode(), $e);
        }
    }

    /**
     * @param ContainerInterface $container
     * @param string $requestedName
     *
     * @return array|null
     */
    private function getConfig(ContainerInterface $container, string $requestedName): ?array
    {
        try {
            $config = $container->get('config')['db'] ?? [];
        } catch (NotFoundExceptionInterface|ContainerExceptionInterface) {
            return null;
        }

        if ($requestedName === 'db') {
            $requestedName = $config['default_connection'];
        }

        return $config['connections'][$requestedName] ?? null;
    }
}
